{{# def.definitions }}

{{ /**
    * schema compilation (render) time:
    * it = { schema, RULES, _validate, opts }
    * it.validate - this template function,
    *   it is used recursively to generate code for subschemas
    *
    * runtime:
    * "validate" is a variable name to which this function will be assigned
    * validateRef etc. are defined in the parent scope in index.js
    */ }}

{{? it.isRoot}}
  {{
    var $root = it.isRoot
      , $lvl = it.level = 0
      , $dataLvl = it.dataLevel = 0
      , $data = 'data';
    it.baseId = it.resolve.fullPath(it.schema.id);
    delete it.isRoot;
    it.wasRoot = true;
  }}

  validate = function (data, dataPath) {
    'use strict';
    validate.errors = null;{{ /* don't edit, used in replace */ }}
    var errors = 0;        {{ /* don't edit, used in replace */ }}
{{??}}
  {{? it.opts._debug }} console.log('validate dataPath:', dataPath); {{?}}

  {{ 
    var $lvl = it.level
      , $dataLvl = it.dataLevel
      , $data = 'data' + ($dataLvl || '');

    if (it.schema.id) it.baseId = it.resolve.url(it.baseId, it.schema.id);
    delete it.wasRoot;
  }}

  var errs_{{=$lvl}} = errors;
{{?}}

{{
  var $valid = 'valid' + $lvl
    , $breakOnError = !it.opts.allErrors
    , $closingBraces1 = ''
    , $closingBraces2 = '';

  var $typeSchema = it.schema.type;
}}

{{~ it.RULES:$rulesGroup }}
  {{? $shouldUseGroup($rulesGroup) }}
    {{? $rulesGroup.type }}
      if ({{= it.util.checkDataType($rulesGroup.type, $data) }}) {
    {{?}}
      {{~ $rulesGroup.rules:$rule }}
        {{? $shouldUseRule($rule) }}
          {{= $rule.code(it) }}
          {{? $breakOnError }}
            {{ $closingBraces1 += '}'; }}
          {{?}}
        {{?}}
      {{~}}
      {{? $breakOnError }}
        {{= $closingBraces1 }}
        {{ $closingBraces1 = ''; }}
      {{?}}
    {{? $rulesGroup.type }}
      }
      {{? $typeSchema && $typeSchema === $rulesGroup.type }}
        {{ var $typeChecked = true; }}
        else {
          {{# def.error:'type' }}
        }
      {{?}}
    {{?}}

    {{? $breakOnError }}
      if (errors === {{?$root}}0{{??}}errs_{{=$lvl}}{{?}}) {
      {{ $closingBraces2 += '}'; }}
    {{?}}
  {{?}}
{{~}}

{{? $typeSchema && !$typeChecked  }}
  {{
    var $schemaPath = it.schemaPath + '.type'
      , $isArray = Array.isArray($typeSchema)
      , $method = $isArray ? 'checkDataTypes' : 'checkDataType';
  }}

  if ({{= it.util[$method]($typeSchema, $data, true) }}) {
    {{# def.error:'type' }}
  }
{{?}}

{{? $breakOnError }} {{= $closingBraces2 }} {{?}}

{{? $root }}
    return errors === 0;{{ /* don't edit, used in replace */ }}
  }
{{??}}
    var {{=$valid}} = errors === errs_{{=$lvl}};
{{?}}

{{# def.cleanUp }}

{{? $root && $breakOnError }}
  {{# def.cleanUpVarErrors }}
{{?}}

{{
  function $shouldUseGroup($rulesGroup) {
    return $rulesGroup.rules.some(function ($rule) {
      return $shouldUseRule($rule);
    });
  }

  function $shouldUseRule($rule) {
    return it.schema[$rule.keyword] !== undefined ||
           ( $rule.keyword == 'properties' &&
             ( it.schema.additionalProperties !== undefined ||
               ( it.schema.patternProperties &&
                 Object.keys(it.schema.patternProperties).length )));
  }
}}
